import { declare } from '@babel/helper-plugin-utils';
import syntaxImportSourcePhase from '@babel/plugin-syntax-import-source';
import { buildParallelStaticImports, importToPlatformApi } from '@babel/helper-import-to-platform-api';

var index = declare(api => {
  const {
    types: t,
    template
  } = api;
  api.assertVersion("8.0.0-alpha.14");
  const targets = api.targets();
  let helperESM;
  let helperCJS;
  const transformers = {
    webFetch: fetch => template.expression.ast`WebAssembly.compileStreaming(${fetch})`,
    nodeFsSync: read => template.expression.ast`new WebAssembly.Module(${read})`,
    nodeFsAsync: template.expression`WebAssembly.compile`
  };
  const getHelper = file => {
    const modules = file.get("@babel/plugin-transform-modules-*");
    if (modules === "commonjs") {
      return helperCJS ??= importToPlatformApi(targets, transformers, true);
    }
    if (modules == null) {
      return helperESM ??= importToPlatformApi(targets, transformers, false);
    }
    throw new Error(`@babel/plugin-proposal-import-wasm-source can only be used when not ` + `compiling modules, or when compiling them to CommonJS.`);
  };
  return {
    name: "proposal-import-wasm-source",
    inherits: syntaxImportSourcePhase,
    visitor: {
      Program(path) {
        if (path.node.sourceType !== "module") return;
        const helper = getHelper(this.file);
        const t2 = t;
        const data = [];
        for (const decl of path.get("body")) {
          if (!decl.isImportDeclaration({
            phase: "source"
          })) continue;
          if (decl.node.attributes?.length || decl.node.assertions?.length) {
            throw path.buildCodeFrameError("`import source` with import attributes cannot be compiled.");
          }
          const specifier = decl.node.specifiers[0];
          t2.assertImportDefaultSpecifier(specifier);
          data.push({
            id: specifier.local,
            fetch: helper.buildFetch(decl.node.source, path)
          });
          decl.remove();
        }
        const decl = buildParallelStaticImports(data, helper.needsAwait);
        if (decl) path.unshiftContainer("body", decl);
      },
      ImportExpression(path) {
        if (path.node.phase !== "source") return;
        if (path.node.options) {
          throw path.buildCodeFrameError("`import.source` with an options bag cannot be compiled.");
        }
        path.replaceWith(getHelper(this.file).buildFetchAsync(path.node.source, path));
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
